# Components

In React basically everything that is *viewable* is a component (or can be).

The idea of a component is that is reusable and can contain other components

Because everything is a component there is a `root component` which is located at `frontend/src/index.tsx`

As you can see in the file it consists of some other components for routing, state management, login, ...

For detailed information about the app is instantiated see [workflow](../overview/workflow.md)

Note that there is a component `App` which used inside the root component and is located in `frontend/src/app.tsx`

This represents our app and e.g. contains all components

*There is no rule how big or small a component should be*

If you want to create a component you should differentiate between a new site/page for the app or just a smaller part (we call it a view)

Sites normally have a separate url and e.g. can be bookmarked in the browser, views are sub-components of site-components e.g. the `consoleView`
If you want to create a new site-component see [site component tutorial](siteComponents.md)

The next question you should answer is if you need [hooks](https://reactjs.org/docs/react-component.html#the-component-lifecycle) or not

*Hooks are functions that get called if the component changed e.g. props updated, the component was created/mounted, ...*

If you don't need any hook you can go with stateless components

Else you want a normal component

*Because we don't use the state static feels more right than stateless component as description. For the reason why we don't use the react component state see [redux](redux.md)*


## Props

Every component has properties (props)

Props are data that is received by our component (passed down from other components)

The props are normally defined as an interface in the same file as the component which uses the props

How you declare props and how to use them is described in the next section

## Stateless (static) components

a small example for a static component looks like this

```ts
//file testComp.tsx
import * as React from "react";

export interface MyProps {
  /**
   * this is required
   */
  readonly	name: string
  /**
   * this is an optional prop, so we need to check if it specified or not
   */
  readonly age?: number
}

export const TestComp: React.SFC<any> = (props: MyProps) => {

  return (
    <div>
      <h1>Hello {props.name}</h1>
      {
        props.age && //check if the prop is specified or not
          <div>
            your age is <span className="nice" style={{fontStyle: 'italic'}}>{props.age}</span>
          </div>
      }
    </div>
  )
}

//file site.tsx
...
render(): JSX.Element {
	<TestComp name="janis" age={22}></TestComp>

	<TestComp name="max"></TestComp>
}
...

```

So `MyProps` defines the props that our `TestComp` expect

`Age` is optional

Because we don't need and hooks are state the component itself is just a function returning the *html* code

All components have the extension `.tsx` so that the typescript compiler knows that this is not a *normal* typescript file
Actually react uses a domain specific language (DSL) called jsx to transform the code under the `render` function. It looks like html with the `{}` sections that is then translated into React.render(...) calls, thus react uses purely js under the hood

To be able to allow full js/ts support the `{}` sections are used so everything inside `{}` is interpret as js/ts code and evaluated

The `props.age &&` is used to check if the age property is set. If this is the case the `props.age &&` part becomes true and the right hand side is returned

If not null (or undefined?) is returned and nothing is returned


Of course the component is reactive! So if our component in the `site.tsx` would look like this

```ts
render(): JSX.Element {
	<TestComp name={name}" age={22}></TestComp>
}
```

and `name` would be a variable that is changed when a button is clicked the new value will be propagated down to our `TestComp` and react would render our `TestComp` again with the `name` prop set to the new value


If you want to do something inside the static component (e.g. call a function) you can pass a function as a prop!
It's also possible to customize/extend some components e.g.


```ts
import * as React from "react";
import {ButtonProps, Button} from "semantic-ui-react";

export interface MyProps {
}

export const TestComp: React.SFC<any> = (props: MyProps & ButtonProps) => {
	return (
		<div>
			<Button
				content={translate(TranslateIds.cancel)}
				icon='cancel'
				labelPosition='left'
				{...props}
			>
			</Button>
		</div>
	)
}
```

Here we use props from `semantic-ui-react` button component and set some props to static values

Note that we use `{...props}` [js object spread operator](https://developer.mozilla.org/de/docs/Web/JavaScript/Reference/Operators/Spread_syntax) so that all props are propagated down to the `Button` component

If we write

```ts
<TestComp icon="x"></TestComp>
```

this will overwrite the `cancel` icon because the properties contains `{icon: "x"}` and this will overwrite the previous `icon='cancel'`

If we would use the spread operator before our static values we would **not** redefine the property values because then `icon='cancel'` will overwrite any value for icon in the props


Another neat trick is [object decomposition](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/Destructuring_assignment) for props

```ts
import * as React from "react";
import {Icon, Popup, PopupProps} from "semantic-ui-react";

export interface MyProps {
  readonly defaultText: string | JSX.Element
}

export const HelpPopup: React.SFC<any> = (props: MyProps & PopupProps) => {

  const {defaultText, ...popupProps} = props

  return (
    <div style={{display: 'inline-block'}}>
      <Popup
        trigger={
          <Icon name="question circle" />
        }
        content={defaultText}
        {...popupProps}
      />
    </div>
  )
}
```

Here we we split the props into two parts: our props and the `PopupProps` from semantic ui
Because the props are the union type of both we cannot do `{...props}` inside the Popup because the Popup component don' have a property named `defaultText` [which would lead to a warning](https://reactjs.org/warnings/unknown-prop.html)

So we split the props first into the two parts then then we use them

---

And what is the `any` in `React.SFC<any>` ? No idea...

## components

So *normal* component *can* use hooks (compared to static components)
but most of the time they are not really needed

so a small example could be

```ts

import * as React from "react";
import {ButtonProps, Button} from "semantic-ui-react";
import {translate} from "../../../i18n/translationHelper";
import {TranslateIds} from "../../../i18n/ids";

export interface MyProps {
}

class CancelButton extends React.Component<MyProps & ButtonProps, any> {
	foo1():void {
  }

	foo2 = () =>  {
    return 1
  }

	foo3: () => number = () =>  {
    return 1
  }
  
  componentWillMount(): void {
    //dispatch
    this.foo()
  }

  render(): JSX.Element {
    return (
      <div>
        <Button
          content={translate(TranslateIds.cancel)}
          icon='cancel'
          labelPosition='left'
          {...this.props}
        >
        </Button>
      </div>
    )
  }
}

export default CancelButton
```

Note that we use `this.props` instead of `props` (like in the static component) because we are not inside a class

The first argument of `React.Component<TProps, TState>` is the type for the props of the component `TProps`
The second argument is for the `State` but as stated above we don't ever use the local state of a component, see [redux](redux.md) why

The `render` describes the visual appearance of the component

`componentWillMount` is called before the component is mounted (added to the DOM)

There are a few other hooks you can use and of course you can define own functions (`foo`) like in normal classes and use them


As we know from static components all components are reactive so they are re-rendered (if needed)
For every re-render the render function is called or the whole component needs to be re-created (depending on how *deep* the change was)

This can lead to memory leaks because if we use arrow/anonymous functions e.g.

```ts
render(): JSX.Element {
    return (
      <div onClick={() => /* do some stuff */}></div>
}
```

will re-create the function every time the component re-renders

[TODO ??]
Actually we haven't payed any attention to this because we wanted to get a stable version of YAPEX but this could be definitely improved over the whole app
A solution would be to use real functions like the `foo` ones

We also think that the re-creation of components can lead to memory leaks so not all version if `foo` are correct [TODO which one is correct??]



## More complex components

Before we save some basic components where we specified the in-props manually ... but where the props come from (because the parent components pass down the props but where the parent components get the props/data from)?

The whole data for app is stored in one object, you can read more about this in [redux](redux.md).

Because the components can't be explained separately from redux you might read redux and this explanation again until you get it

So a more complex component has also props but has some boiler plate code and looks like this

```ts
export interface MyProps {
	readonly test: string
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
		test: props.test,
	}
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

	//setXXXFunc //some redux action

}, dispatch)

const stateProps = returntypeof(mapStateToProps)
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ConsolePanelView extends React.Component<Props, any> {
  render(): JSX.Element {

		//...
		//this.props.setXXXFunc()
		//this.props.test

		return (
			<div></div>
		)
	}
}

export default connect(mapStateToProps, mapDispatchToProps)(ConsolePanelView) //don't forget to call connect !!
```

So there are these 2 magical functions `mapStateToProps` and `mapDispatchToProps`

The component props are somehow the union of the return types of the 2 functions

*If you don't get any props or something is undefined then you probably missed to call the `connect()` function on the component*


`mapStateToProps` is used to calculate the props for the component
we get the `RootState` and `MyProps`

`MyProps` we already know from the *easy* components
`RootState` represents the whole state of the app (see [redux](redux.md))

`mapStateToProps` returns an object where we selected the properties we want inside our props
Of course we don't need (and want) the whole state of the app in our component! So we grab only the parts we need and combine them with the props (`MyProps`) we get from the parent component into one object

The object (the new props) returned by `mapStateToProps` is then accessible inside the component via `this.props` e.g. `this.props.test`


*Keep in mind that `mapStateToProps` is called (and thus the component re-renders) via redux every time the used properties from the `RootState` are changed. So if we use the root state itself every change to the root state will lead to a re-render of our component*


`mapDispatchToProps` defines the functions (redux actions) that are used inside our component. Basically we use the actions to change the state of our app. Every functions specified (e.g. foo) there is accessible via `this.props` e.g. `this.props.foo(...)`

Because we don't use the component local state we use `any` in `React.Component<TProps, TState>`
Another side effect of this is that we don't need to worry about re-rendering that much because redux will trigger the re-render for us every time the used data (by our component) is changed


## Further hints

Some hooks can be async e.g.

```ts
async componentWillMount(): Promise<void> {

    //this.props.data is null
    try {
      await this.props.loadData(...)
    } catch (err) {
      //TODo error
      return
    }

    //this.props.data is set e.g. to list: [...]
}
```

which is useful if you want to await data

What's also important is that if you await inside a hook (only tested in componentWillMount) this will recompute the props so after the await finished (and the action somehow modified the props via the RootState) the new values for props are already there!

## Reuseable components

If you want to write a truly reusable component make sure you

- don't use the `RootState` and
- `mapDispatchToProps` is empty

Everything should be handled via the `MyProps` this way the data source is moved to the parent component and thus your component don't have any dependency

## Next (modify state)

A lot of components need to modify the state of the app, actually even a button click leads to a state change

see [adding state changes](addingStateChanges.md)
